import pandas as pd
import networkx as nx
import matplotlib.pyplot as plt
import sys

def bellmanford(graph, srcid):
    node_dict = dict()        # list of nodes to work on
    dist_to_src = dict()      # distance to source on shortest paths
    prec_node_to_dst = dict() # preceding node to destination on shortest paths
    for node in graph.nodes:
        dist_to_src[node] = float('inf')
        prec_node_to_dst[node] = None
        node_dict[str(node)] = node
    dist_to_src[list(graph.nodes)[srcid]] = 0

    for node_key in node_dict:
        node_u = node_dict[node_key]
        edges_v = graph.out_edges(node_u, data=True)
        for edge in edges_v:
            edge_cost = edge[2]['cost']
            node_v = node_dict[str(edge[1])] 
            distance = dist_to_src[node_u] + edge_cost
            if distance < dist_to_src[node_v]:
                dist_to_src[node_v] = distance
                prec_node_to_dst[node_v] = node_u

    # logic to detect negative cycle ignored
    
    return dist_to_src, prec_node_to_dst

def plot_graph(graph):
    plt.subplot(121)
    pos = nx.spring_layout(graph)
    nx.draw(graph, pos, with_labels=True)
    edge_labels=dict([((u,v,),d['cost']) for u,v,d in graph.edges(data=True)])
    nx.draw_networkx_edge_labels(graph, pos, edge_labels=edge_labels, \
            label_pos=0.3, font_size=7)

if __name__=="__main__":
    edgesDf = pd.read_csv('graph_edges.csv')
    nodesDf = pd.read_csv('graph_nodes.csv')
    
    graph = nx.MultiDiGraph()
    
    for i, node in nodesDf.iterrows():
        graph.add_node(node['id'])
    
    for i,edge in edgesDf.iterrows():
        graph.add_edge(edge['src'], edge['dst'], cost=edge['cost'])
    
    nodeName = 'v1'
    srcid = [i for i,name in enumerate(graph.nodes()) if name == nodeName]
    dist_to_src,prec_node_to_dst = bellmanford(graph, srcid[0])
    print('dist_to_src = ', dist_to_src)
    print('prec_node_to_dst = ', prec_node_to_dst)
    
    plot_graph(graph)

    # show graph. this is blocking
    plt.show()
